import 'dart:math';
import 'dart:ui' as ui;

import 'package:flutter/material.dart';

class ScreenUtil {
  static const Size defaultSize = Size(360, 780);
  static late ScreenUtil _instance;

  /// UI设计中手机尺寸 , dp
  late Size uiSize;

  ///屏幕方向
  late Orientation _orientation;

  ///像素比
  late double _pixelRatio;

  ///文字缩放比
  late double _textScaleFactor;

  ///屏幕宽度
  late double _screenWidth;

  ///屏幕搞定
  late double _screenHeight;

  ///状态栏
  late double _statusBarHeight;

  ///底部安全区
  late double _bottomBarHeight;

  ScreenUtil._();

  factory ScreenUtil() {
    return _instance;
  }

  static void init(
    BoxConstraints constraints, {
    Orientation orientation = Orientation.portrait,
    Size designSize = defaultSize,
  }) {
    _instance = ScreenUtil._()
      ..uiSize = designSize
      .._orientation = orientation
      .._screenWidth = constraints.maxWidth
      .._screenHeight = constraints.maxHeight;

    var window = WidgetsBinding.instance?.window ?? ui.window;
    _instance._pixelRatio = window.devicePixelRatio;
    _instance._statusBarHeight = window.padding.top;
    _instance._bottomBarHeight = window.padding.bottom;
    _instance._textScaleFactor = window.textScaleFactor;
  }

  ///获取屏幕方向
  Orientation get orientation => _orientation;

  ///每个逻辑像素的字体像素数，字体的缩放比例
  double get textScaleFactor => _textScaleFactor;

  ///设备的像素密度
  double get pixelRatio => _pixelRatio;

  ///当前设备宽度
  double get screenWidth => _screenWidth;

  ///当前设备高度
  double get screenHeight => _screenHeight;

  ///状态栏高度
  double get statusBarHeight => _statusBarHeight / _pixelRatio;

  ///底部安全区距离
  double get bottomBarHeight => _bottomBarHeight / _pixelRatio;

  ///实际尺寸与UI设计的宽度比例
  double get scaleWidth => _screenWidth / uiSize.width;

  ///实际尺寸与UI设计的高度比例
  double get scaleHeight => _screenHeight / uiSize.height;

  ///文本缩放比，取宽高比例最小值
  double get scaleText => min(scaleWidth, scaleHeight);

  /// 根据UI设计的设备宽度适配
  double setWidth(num width) => width * scaleWidth;

  /// 根据UI设计的设备高度适配
  double setHeight(num height) => height * scaleHeight;

  ///根据宽度或高度中的较小值进行适配
  double radius(num r) => r * scaleText;

  ///字体大小适配方法
  double setSp(num fontSize) => fontSize * scaleText;
}
