import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:wise_util/res/wise_color.dart';
import 'package:wise_util/res/wise_localizations.dart';
import 'package:wise_util/res/wise_style.dart';

class WisePullToRefresh extends StatefulWidget {
  final bool enablePullDown;

  final bool enablePullUp;

  final VoidCallback? onRefresh;

  final VoidCallback? onLoading;

  final Widget child;

  WisePullToRefresh({
    required this.child,
    this.enablePullUp = true,
    this.enablePullDown = true,
    this.onRefresh,
    this.onLoading,
  });

  @override
  State<StatefulWidget> createState() => _WisePullToRefreshState();
}

class _WisePullToRefreshState extends State<WisePullToRefresh> {
  final RefreshController refreshController =
      RefreshController(initialRefresh: false);

  @override
  Widget build(BuildContext context) {
    WiseString strings =
        WiseLocalizations.of(context)?.currentLocalization ?? EnWiseString();
    return SmartRefresher(
      controller: refreshController,
      header: ClassicHeader(
        textStyle: WiseStyle.textStyleSecondText(),
        completeIcon: Icon(Icons.done, color: WiseColor.colorPrimary()),
        idleIcon: Icon(Icons.arrow_downward, color: WiseColor.colorPrimary()),
        releaseIcon: Icon(Icons.refresh, color: WiseColor.colorPrimary()),
        failedIcon: Icon(Icons.error, color: WiseColor.colorPrimary()),
        releaseText: strings.refreshRelease,
        refreshingText: strings.refreshRefreshing,
        failedText: strings.refreshFailed,
        completeText: strings.refreshCompleted,
        idleText: strings.refreshIdle,
      ),
      // footer: ClassicFooter(
      //   textStyle: WiseStyle.textStyleSecondText(),
      //   // loadingIcon:
      //   //     Icon(Icons.refresh, color: WiseColor.colorPrimary(), size: 30.w),
      // ),
      enablePullDown: widget.enablePullDown,
      enablePullUp: widget.enablePullUp,
      onRefresh: _onRefresh,
      onLoading: _onLoading23,
      child: widget.child,
    );
  }

  void _onRefresh() async {
    try {
      print("_onRefresh");
      if (null != widget.onRefresh) widget.onRefresh!();
      await Future.delayed(Duration(seconds: 2));
      refreshController.refreshCompleted();
    } catch (e) {
      refreshController.refreshFailed();
    }
  }

  void _onLoading23() async {
    try {
      print("_onLoading");
      if (null != widget.onLoading) widget.onLoading!();
      await Future.delayed(Duration(seconds: 5));
      throw Exception("123");
      refreshController.loadComplete();
    } catch (e) {
      refreshController.loadFailed();
    }
  }
}
